import { Body, Controller, Get, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionDiaService } from './institucion-dia.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { UpdateInstitucionDiaDto } from './dto/input/update.dto';
import { InstitucionCarreaOutputDto } from './dto/output/institucion-carrera.dto';

@Controller('institucion-dia')
@ApiTags('institucion-dia')
export class InstitucionDiaController {
  constructor(private institucionDiaService: InstitucionDiaService) {}

  @Serealize(InstitucionCarreaOutputDto)
  @Get()
  @ApiOperation({
    description: 'Endpoint que retorna todos los días de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  dias(@Query() query: IdInstitucionDto) {
    return this.institucionDiaService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Put()
  @ApiOperation({
    description:
      'Endpoint que actualiza la información de un día de una institución.',
  })
  @ApiBody({
    description:
      'Todas las variables a excepción de id_institucion_dia son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion_dia: 217,
          _activo: true,
          _hora_fin: '17:45',
          _hora_inicio: '09:00',
        },
      },
    },
  })
  update(@Body() body: UpdateInstitucionDiaDto) {
    return this.institucionDiaService.update(body);
  }
}
