import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Infraccion } from './entity/infraccion.entity';
import { InstitucionInfraccion } from './entity/institucion-infraccion.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionInfraccionService {
  constructor(
    @InjectRepository(Infraccion)
    private infraccionRepository: Repository<Infraccion>,
    @InjectRepository(InstitucionInfraccion)
    private institucionInfraccionRepository: Repository<InstitucionInfraccion>,
    private institucionService: InstitucionService,
  ) {}

  create(infraccion: string) {
    return this.infraccionRepository
      .findOne({ infraccion })
      .then((existeInfraccion) => {
        if (existeInfraccion)
          throw new ConflictException('Ya existe esta infracción.');
        return this.infraccionRepository.save(
          this.infraccionRepository.create({ infraccion }),
        );
      })
      .then(async (infraccion) => {
        const instituciones = await this.institucionService.findAll();

        for (let i = 0; i < instituciones.length; i++)
          await this.institucionInfraccionRepository.save(
            this.institucionInfraccionRepository.create({
              infraccion,
              institucion: instituciones[i],
            }),
          );
        return { message: 'Se creó correctamente una nueva infracción.' };
      });
  }

  findAll() {
    return this.infraccionRepository.find();
  }

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.institucionInfraccionRepository.find({ institucion }),
      );
  }

  findById(id_institucion_infraccion: number) {
    return this.institucionInfraccionRepository
      .findOne({ id_institucion_infraccion })
      .then((institucionInfraccion) => {
        if (!institucionInfraccion)
          throw new NotFoundException('No existe esta institucion infracción.');
        return institucionInfraccion;
      });
  }

  update(attrs: Partial<InstitucionInfraccion>) {
    return this.findById(attrs.id_institucion_infraccion)
      .then((institucionInfraccion) => {
        Object.assign(institucionInfraccion, attrs);
        return this.institucionInfraccionRepository.save(institucionInfraccion);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
