import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionTipoEntrada } from './entity/institucion-tipo-entrada.entity';
import { TipoEntrada } from './entity/tipo-entrada.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionTipoEntradaService {
  constructor(
    @InjectRepository(InstitucionTipoEntrada)
    private institucionTipoEntradaRepository: Repository<InstitucionTipoEntrada>,
    @InjectRepository(TipoEntrada)
    private tipoEntradaRepository: Repository<TipoEntrada>,
    private institucionService: InstitucionService,
  ) {}

  async create(tipo_entrada: string) {
    return this.tipoEntradaRepository
      .findOne({ tipo_entrada })
      .then((existeTipoEntrada) => {
        if (existeTipoEntrada)
          throw new ConflictException('Ya existe este tipo de entrada.');
        return this.tipoEntradaRepository.save(
          this.tipoEntradaRepository.create({
            tipo_entrada,
          }),
        );
      })
      .then(async (tipoEntrada) => {
        const instituciones = await this.institucionService.findAll();

        for (let i = 0; i < instituciones.length; i++)
          await this.institucionTipoEntradaRepository.save(
            this.institucionTipoEntradaRepository.create({
              institucion: instituciones[i],
              tipoEntrada,
            }),
          );
        return {
          message: 'Se creó correctamente un nuevo tipo de entrada.',
        };
      });
  }

  findAll() {
    return this.tipoEntradaRepository.find();
  }

  async findAllByIdInstitucion(id_institucion: number, mostrar = false) {
    const institucion = await this.institucionService.findById(id_institucion);
    const busqueda: { institucion: Institucion; mostrar?: boolean } = {
      institucion,
    };

    if (mostrar) busqueda.mostrar = mostrar;
    return this.institucionTipoEntradaRepository.find(busqueda);
  }

  findById(id_institucion_tipo_entrada: number) {
    return this.institucionTipoEntradaRepository
      .findOne({
        id_institucion_tipo_entrada,
      })
      .then((institucionTipoEntrada) => {
        if (!institucionTipoEntrada)
          throw new NotFoundException(
            'No existe esta institucion tipo entrada.',
          );
        return institucionTipoEntrada;
      });
  }

  findTipoEntradaById(id_tipo_entrada: number) {
    return this.tipoEntradaRepository
      .findOne({ id_tipo_entrada })
      .then((tipoEntrada) => {
        if (!tipoEntrada)
          throw new NotFoundException('No existe este tipo de entrada.');
        return tipoEntrada;
      });
  }

  findTipoEntradaByTipoEntrada(tipo_entrada: string, validarNoExiste = true) {
    return this.tipoEntradaRepository
      .findOne({ tipo_entrada })
      .then((tipoEntrada) => {
        if (validarNoExiste && !tipoEntrada)
          throw new NotFoundException('No existe este tipo de entrada.');
        return tipoEntrada;
      });
  }

  update(attrs: Partial<InstitucionTipoEntrada>) {
    return this.findById(attrs.id_institucion_tipo_entrada)
      .then((institucionTipoEntrada) => {
        Object.assign(institucionTipoEntrada, attrs);
        return this.institucionTipoEntradaRepository.save(
          institucionTipoEntrada,
        );
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
