import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Institucion } from './entity/institucion.entity';

@Injectable()
export class InstitucionService {
  constructor(
    @InjectRepository(Institucion) private repository: Repository<Institucion>,
  ) {}

  findAll(activo = false) {
    const busqueda: { activo?: boolean } = {};

    if (activo) busqueda.activo = activo;
    return this.repository.find(busqueda);
  }

  findById(id_institucion: number) {
    return this.repository.findOne({ id_institucion }).then((institucion) => {
      if (!institucion)
        throw new NotFoundException('No existe este id institución.');
      return institucion;
    });
  }

  update(attrs: Partial<Institucion>) {
    return this.findById(attrs.id_institucion)
      .then((institucion) => {
        Object.assign(institucion, attrs);
        return this.repository.save(institucion);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
