import * as moment from 'moment';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Motivo } from './entity/motivo.entity';
import { EquipoService } from '../equipo/equipo.service';
import { OperadorService } from '../operador/operador.service';
import { StatusService } from '../status/status.service';
import { Equipo } from 'src/equipo/entity/equipo.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { Status } from 'src/status/entity/status.entity';

@Injectable()
export class MotivoService {
  constructor(
    @InjectRepository(Motivo) private repository: Repository<Motivo>,
    private equipoService: EquipoService,
    private operadorService: OperadorService,
    private statusService: StatusService,
  ) {}

  async create(
    equipo: Equipo,
    id_operador: number | Operador,
    id_status: number | Status,
    motivo: string,
  ) {
    const operador =
      typeof id_operador === 'number'
        ? await this.operadorService.findById(id_operador)
        : id_operador;
    const status =
      typeof id_status === 'number'
        ? await this.statusService.findById(id_status)
        : id_status;

    return this.repository.save(
      this.repository.create({
        equipo,
        fecha_creacion: moment().toDate(),
        motivo,
        operador,
        status,
      }),
    );
  }

  findAllByIdEquipo(id_equipo: number, pagina: number) {
    return this.equipoService.findById(id_equipo).then((equipo) =>
      this.repository.findAndCount({
        where: { equipo },
        skip: (pagina - 1) * 25,
        take: 25,
      }),
    );
  }
}
