import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { OperadorService } from './operador.service';
import { CreateOperadorDto } from './dto/input/create.dto';
import { OperadoresDto } from './dto/input/operadores.dto';
import { UpdateOperadorDto } from './dto/input/update.dto';
import { OperadorDto } from './dto/input/operador.dto';
import { OperadorOutputDto } from './dto/output/operador.dto';

@Controller('operador')
@ApiTags('operador')
export class OperadorController {
  constructor(private operadorService: OperadorService) {}

  @Post()
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea un operador.' })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: {
      ejemplo: {
        value: {
          id_institucion: 200,
          id_tipo_usuario: 4,
          operador: '',
          password: '',
        },
      },
    },
  })
  create(@Body() body: CreateOperadorDto) {
    return this.operadorService.create(
      body.id_institucion,
      body.id_tipo_usuario,
      body.operador,
      body.password,
    );
  }

  @Serealize(OperadorOutputDto)
  @Get('operador')
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna la información de un operador.',
  })
  @ApiQuery({
    description: 'Id del operador.',
    name: 'id_operador',
    type: 'string',
  })
  operador(@Query() query: OperadorDto) {
    return this.operadorService.findById(parseInt(query.id_operador));
  }

  @Serealize(OperadorOutputDto)
  @Get('operadores')
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna 25 operadores dependiendo de la página en la que se encuentra el usuario y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id de la institución que se quiere usar como filtro.',
    name: 'id_institucion',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo usuario que se quiere usar como filtro.',
    name: 'id_tipo_usuario',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Operador que se quiere buscar.',
    name: 'operador',
    type: 'string',
    required: false,
  })
  operadores(@Query() query: OperadoresDto) {
    return this.operadorService.findAll(query);
  }

  // @Get('reporte')
  // @UseGuards(AuthGuard('jwt'))
  // reporte() {}

  @Put()
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que actualiza la información de un operador.',
  })
  @ApiBody({
    description:
      'Todas las variables a excepción de id_operador son opcionales.',
    examples: {
      ejemplo: { value: { id_operador: 3, activo: true, password: '' } },
    },
  })
  update(@Body() body: UpdateOperadorDto) {
    return this.operadorService.update(body);
  }
}
